import { Injectable } from "@angular/core";
import { NavigationEnd, NavigationStart, NavigationCancel, Router, RouterEvent } from "@angular/router";
import { DARK_THEME, NbThemeService } from "@nebular/theme";
import { Course } from "app/features/courses/models/course";
import { Term } from "app/features/modules/models/term";
import { TermService } from "app/features/modules/services/term.service";
import { NotificationService } from "app/shared/services/notification.service";
import { BehaviorSubject, of, Subject } from "rxjs";
import { Observable } from "rxjs/Observable";
import { filter, flatMap, map, take } from "rxjs/operators";
import { SUBATO_THEME } from "../core.module";
import { SubatoPrincipal } from "../models/principal/subato-principal";
import { PublicConfig } from "../models/public-config";
import { AuthService } from "./auth.service";
import { ConfigService } from "./config.service";


@Injectable({
    providedIn: 'root'
})
export class AppStateService {
    private _init: Subject<boolean>;
    init$: Observable<boolean>;

    currentUrl: string = null;
    prevUrl: string = null;
    private _prevUrl: Subject<string>;
    prevUrl$: Observable<string>;

    private _term: Subject<Term>
    term$: Observable<Term>
    private currentTerm: Term | null;
    terms: Term[] = [];

    principal: SubatoPrincipal | null = null;

    private _darkMode: Subject<boolean>
    darkMode$: Observable<boolean>
    darkMode: boolean;

    private _courseId: Subject<Course>
    courseId$: Observable<Course>
    courseId: Course

    config: PublicConfig;

    constructor(
        private router: Router,
        private termService: TermService,
        private authService: AuthService,
        private themeService: NbThemeService,
        private configService: ConfigService,
        private notificationService: NotificationService
    ) {
        this._init = new BehaviorSubject<boolean>(false);
        this.init$ = this._init.asObservable();

        this._term = new BehaviorSubject<Term>(null);
        this.term$ = this._term.asObservable();

        this._darkMode = new BehaviorSubject<boolean>(false);
        this.darkMode$ = this._darkMode.asObservable();
        this.darkMode$.subscribe(darkMode => {
            this.darkMode = darkMode;
            this.applyTheme();
        });

        this._courseId = new BehaviorSubject<Course>(null);
        this.courseId$ = this._courseId.asObservable();

        this.term$.subscribe(term => this.currentTerm = term);

        this._prevUrl = new BehaviorSubject<string>(null);
        this.prevUrl$ = this._prevUrl.asObservable();
        this.prevUrl$.subscribe(url => this.prevUrl = url);

        this.authService.principal$.subscribe(principal => this.principal = principal);

        this.router.events
            .pipe(filter((event: RouterEvent) => event instanceof NavigationStart))
            .subscribe((event: NavigationStart) => {
                var prevUrl = this.currentUrl
                this._prevUrl.next(prevUrl);
                this.currentUrl = event.url;
            });

        this.router.events
            .pipe(filter((event: RouterEvent) => event instanceof NavigationCancel))
            .subscribe((event: NavigationCancel) => {
                this.currentUrl = this.prevUrl;
            });

        this.loadDarkMode();
    }

    setInit(val: boolean): void {
        this._init.next(val);
    }

    loadDarkMode() {
        let darkMode = localStorage.getItem("darkMode") == "true"
        this._darkMode.next(darkMode);
    }

    applyTheme() {
        this.themeService.changeTheme(this.darkMode ? DARK_THEME.name : SUBATO_THEME.name);
    }

    setDarkMode(val: boolean) {
        localStorage.setItem("darkMode", val ? "true" : "false");
        this._darkMode.next(val);
    }

    setCourse(course: Course) {
        this.courseId = course;

        if (course?.term != null) {
            this.setTerm(course.term);
        }

        this._courseId.next(course);
    }

    public getCurrentTerm(): Term | null {
        return this.currentTerm;
    }

    public loadConfig(): Observable<PublicConfig> {
        return this.configService.getConfig()
            .pipe(
                map(config => {
                    this.config = config;
                    return config;
                })
            );
    }

    public loadTerm(): Observable<Term> {
        return this.termService.getAll()
            .pipe(
                map(terms => {
                    this.terms = terms;

                    if (terms.length != 0) {
                        let termId = localStorage.getItem('termId') != null ? parseInt(localStorage.getItem('termId')) : null;

                        var term;
                        if (termId != null) {
                            term = terms.find(t => t.id == termId);
                        }

                        term = term ?? terms[0];
                        this.setTerm(term);
                        return term;
                    }

                    return null;
                })
            )
    }

    public setTerm(term: Term | null, updateRoute: boolean = false): void {
        if (term?.id == this.currentTerm?.id) {
            return;
        }

        this._term.next(term);
        localStorage.setItem('termId', term?.id?.toString());
        if (!updateRoute) return;

        if (this.currentUrl != '/course') {
            if (this.principal) {
                this.router.navigate(['/', 'dashboard']);
            } else {
                this.router.navigate(['/', 'course']);
            }
        }
    }
}