import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { User } from 'app/core/models/user';
import { CourseUnit } from 'app/features/courses/models/course-unit';
import { CourseUnitService } from 'app/features/courses/services/course-unit.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { Subject } from 'rxjs';
import { debounceTime, takeUntil } from 'rxjs/operators';

@Component({
  selector: 'ngx-course-unit-participant-selection',
  templateUrl: './course-unit-participant-selection.component.html',
  styleUrls: ['./course-unit-participant-selection.component.scss']
})
export class CourseUnitParticipantSelectionComponent extends BaseComponent implements OnInit {
  @Input() courseUnit: CourseUnit;

  @Output() participantSelected: EventEmitter<User> = new EventEmitter<User>();
  
  selectedUser: User | null = null;
  allParticipants: User[] = [];
  participants: User[] = [];

  loading: boolean = false;
  error: BackendError;

  query: string;
  private queryChanged: Subject<string> = new Subject<string>();

  constructor(
    private courseUnitService: CourseUnitService
  ) { 
    super();
  }

  ngOnInit(): void {
    this.loading = true;
    this.courseUnitService.getParticipants(this.courseUnit.id)
      .pipe(
        takeUntil(this.unsubscribe),
      )
      .subscribe(participants => {
        this.allParticipants = participants;
        this.participants = participants;
        if(this.participants.length !== 0) {
          this.selectUser(this.participants[0]);
        }

        this.loading = false;
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });

      this.queryChanged
      .pipe(
        takeUntil(this.unsubscribe),
        debounceTime(500),
      )
      .subscribe(query => {
        this.query = query;
        this.filterUsers(query);
      });
  }

  filterUsers(query: string) {
    this.participants = this.allParticipants.filter(
      p => p.name.toLowerCase().indexOf(query.toLowerCase()) != -1 ||
           (p.loginName != null && p.loginName.toLowerCase().indexOf(query.toLowerCase()) != -1)
    );

    if(this.participants.length !== 0) {
      this.selectUser(this.participants[0]);
    }
  }

  onChange(query) {
    this.queryChanged.next(query);
  }

  selectUser(user: User) {
    this.selectedUser = user;
    this.participantSelected.emit(user);
  }
}
